#include <little/little_rt.h>
#include <glu.h>

#define QUADRICPEER_CLASS	"GLU/QuadricPeer"

static ltl_sym_t float_class = -1;
static ltl_sym_t quadricpeer_class = -1;

LTL_METHOD_DECL(GLU_MetaLibrary_init_0)
{
	LTL_ASSERT_CLASS(float_class,FLOAT_CLASS);
	LTL_ASSERT_CLASS(quadricpeer_class,QUADRICPEER_CLASS);
	return self;	
}

LTL_METHOD_DECL(GLU_MetaConstants_FILL_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,LTL_TYPE_WORD,GLU_FILL);
	return ret;
}

LTL_METHOD_DECL(GLU_MetaConstants_SMOOTH_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,LTL_TYPE_WORD,GLU_SMOOTH);
	return ret;
}

LTL_METHOD_DECL(GLU_MetaFunctions_perspective_4)
{
	union {
		float           f;
		ltl_object_t    *o;
	} v1, v2, v3, v4;
	v1.o = argv[0].fields;
	v2.o = argv[1].fields;
	v3.o = argv[2].fields;
	v4.o = argv[3].fields;
	LTL_CHECK_ARG(0,float_class,"GLU_MetaFunctions_perspective_4");
	LTL_CHECK_ARG(1,float_class,"GLU_MetaFunctions_perspective_4");
	LTL_CHECK_ARG(2,float_class,"GLU_MetaFunctions_perspective_4");
	LTL_CHECK_ARG(3,float_class,"GLU_MetaFunctions_perspective_4");
	printf("perspective = %f %f %f %f\n",v1.f,v2.f,v3.f,v4.f);

	gluPerspective(v1.f,v2.f,v3.f,v4.f);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLU_MetaQuadric_createpeer_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(
		ret,
		quadricpeer_class,
		gluNewQuadric());
	return ret;
}

LTL_METHOD_DECL(GLU_MetaQuadric_deletepeer_1)
{
	LTL_CHECK_ARG(0,quadricpeer_class,"GLU_MetaQuadric_deletepeer_1");
	gluDeleteQuadric((GLUquadricObj *)argv->fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLU_QuadricPeer_drawStyle_1)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLU_MetaQuadric_drawStyle_1");
	printf("quadric draw style = 0x%X\n",argv->fields);
	gluQuadricDrawStyle((GLUquadric *)self->fields,(GLenum)argv->fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLU_QuadricPeer_normals_1)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLU_MetaQuadric_normals_1");
	printf("quadric normals = 0x%X\n",argv->fields);
	gluQuadricNormals((GLUquadric *)self->fields,(GLenum)argv->fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLU_QuadricPeer_sphere_3)
{ 
	union {
		float           f;
		ltl_object_t    *o;
	} val;
	val.o = argv->fields;
	LTL_CHECK_ARG(0,float_class,"GLU_MetaQuadric_sphere_3");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLU_MetaQuadric_sphere_3");
	LTL_CHECK_ARG(2,LTL_TYPE_WORD,"GLU_MetaQuadric_sphere_3");
	gluSphere(
		(GLUquadric *)self->fields,
		(double) val.f,
		(GLint) argv[1].fields,
		(GLint) argv[2].fields);
	return LTL_NIL;
}
