#include <little/little.h>
#include <little/little_rt.h>
#include <little/wrapperutil.h>
#include <glut.h>

#define HMOUSEBUTTON_CLASS "GLUT/HMouseButton"
#define HBITMAPFONT_CLASS "GLUT/HBitmapFont"
#define HSTROKEFONT_CLASS "GLUT/HStrokeFont"
#define HMENU_CLASS "GLUT/HMenu"
#define HRGBCOMP_CLASS "GLUT/HRGBComp"
#define HWINDOW_CLASS "GLUT/HWindow"

static ltl_threadenv_t *glut_threadenv = NULL;
static ltl_object_t *glut_obj = NULL;
static ltl_object_t computer = { LTL_TYPE_VOID, 0, NULL };
static ltl_object_t menustate = { LTL_TYPE_VOID, 0, NULL };
static ltl_object_t timer = { LTL_TYPE_VOID, 0, NULL };

static ltl_object_t *keyboard_callback = NULL;
static ltl_object_t *mouse_callback = NULL;
static ltl_object_t *motion_callback = NULL;
static ltl_object_t *passivemotion_callback = NULL;
static ltl_object_t *painter_callback = NULL;
static ltl_object_t *reshaper_callback = NULL;
static ltl_object_t *entry_callback = NULL;
static ltl_object_t *special_callback = NULL;
static ltl_object_t *spaceballmotion_callback = NULL;
static ltl_object_t *spaceballrotate_callback = NULL;
static ltl_object_t *spaceballbutton_callback = NULL;
static ltl_object_t *buttonbox_callback = NULL;
static ltl_object_t *dials_callback = NULL;
static ltl_object_t *tabletmotion_callback = NULL;
static ltl_object_t *tabletbutton_callback = NULL;
static ltl_object_t *idle_callback = NULL;
static ltl_object_t *menustate_callback = NULL;
static ltl_object_t *timer_callback = NULL;
static ltl_object_t *menu_callback = NULL;

static ltl_sym_t keyboard_mid = -1;
static ltl_sym_t mouse_mid = -1;
static ltl_sym_t motion_mid = -1;
static ltl_sym_t passivemotion_mid = -1;
static ltl_sym_t painter_mid = -1;
static ltl_sym_t reshaper_mid = -1;
static ltl_sym_t entry_mid = -1;
static ltl_sym_t special_mid = -1;
static ltl_sym_t spaceballmotion_mid = -1;
static ltl_sym_t spaceballrotate_mid = -1;
static ltl_sym_t spaceballbutton_mid = -1;
static ltl_sym_t buttonbox_mid = -1;
static ltl_sym_t dials_mid = -1;
static ltl_sym_t tabletmotion_mid = -1;
static ltl_sym_t tabletbutton_mid = -1;
static ltl_sym_t menustate_mid = -1;
static ltl_sym_t idle_mid = -1;
static ltl_sym_t timer_mid = -1;
static ltl_sym_t menu_mid = -1;

static ltl_sym_t char_class = -1;
static ltl_sym_t float_class = -1;
static ltl_sym_t hmousebutton_class = -1;
static ltl_sym_t hbitmapfont_class = -1;
static ltl_sym_t hstrokefont_class = -1;
static ltl_sym_t hmenu_class = -1;
static ltl_sym_t hrgbcomp_class = -1;
static ltl_sym_t hwindow_class = -1;
static ltl_sym_t strbuf_class = -1;

static void displayfunc(void)
{
	printf("display() called\n");
	if ( glut_threadenv && glut_obj ) {
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,painter_callback);
	}
}

static void keyboardfunc(unsigned char c,int x,int y)
{
#ifdef DEBUG
	printf("keyboard(%c,%d,%d) called\n",c,x,y);
#endif
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,c);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,keyboard_callback);
	}
}

static void mousefunc(int button, int state,int x,int y)
{
	printf("mouse(%d,%d,%d,%d) called\n",button,state,x,y);
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,button);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,state);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,mouse_callback);
	}
}

static void motionfunc(int x, int y)
{
#ifdef DEBUG
	printf("motion(%d,%d) called\n",x,y);
#endif
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,motion_callback);
	}
}

static void passivemotionfunc(int x, int y)
{
#ifdef DEBUG
	printf("passivemotion(%d,%d) called\n",x,y);
#endif
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,passivemotion_callback);
	}
}


static void reshapefunc(int w, int h)
{
#ifdef DEBUG
	printf("reshape(%d,%d) called\n",w,h);
#endif
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,w);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,h);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,reshaper_callback);
	}
}

static void entryfunc(int state)
{
#ifdef DEBUG
	printf("entry(%d) called\n",state);
#endif
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,state);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,entry_callback);
	}
}

static void specialfunc(int key,int x,int y)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,key);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,special_callback);
	}
}

static void spaceballmotionfunc(int x,int y,int z)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,z);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,spaceballmotion_callback);
	}
}

static void spaceballrotatefunc(int x,int y,int z)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,z);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,spaceballrotate_callback);
	}
}

static void spaceballbuttonfunc(int button, int state)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,button);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,state);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,spaceballbutton_callback);
	}
}

static void buttonboxfunc(int button, int state)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,button);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,state);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,buttonbox_callback);
	}
}

static void dialsfunc(int dial, int value)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,dial);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,value);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,dials_callback);
	}
}

static void tabletmotionfunc(int x, int y)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,tabletmotion_callback);
	}
}

static void tabletbuttonfunc(int button, int state,int x, int y)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,button);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,state);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,tabletbutton_callback);
	}
}

static void menustatefunc(int status, int x, int y)
{
	if ( glut_threadenv && timer.type ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,status);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,x);
		arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,y);
		ltl_threadenv_invoke_with_frame(glut_threadenv,&menustate,menustate_callback);
	}
}

static void idlefunc()
{
	if ( glut_threadenv && computer.type ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		ltl_threadenv_invoke_with_frame(glut_threadenv,&computer,idle_callback);
	}
}

static void timerfunc(int value)
{
	/*printf("timer(%d) called\n",value);*/
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,value);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,timer_callback);
	}
}

static void menufunc(int item)
{
	if ( glut_threadenv && glut_obj ) {
		ltl_object_t *arg = ltl_threadenv_push(glut_threadenv);
		LTL_OBJECT_SETSPECIAL(arg,LTL_TYPE_WORD,item);
		ltl_threadenv_invoke_with_frame(glut_threadenv,glut_obj,menu_callback);
	}
}

static void registerCallbacks()
{
	glutDisplayFunc(&displayfunc);
	glutKeyboardFunc(&keyboardfunc);
	glutMouseFunc(&mousefunc);
	glutMotionFunc(&motionfunc);
	glutPassiveMotionFunc(&passivemotionfunc);
	glutReshapeFunc(&reshapefunc);
	glutEntryFunc(&entryfunc);
	glutSpecialFunc(&specialfunc);
	glutSpaceballMotionFunc(&spaceballmotionfunc);
	glutSpaceballRotateFunc(&spaceballrotatefunc);
	glutSpaceballButtonFunc(&spaceballbuttonfunc);
	glutButtonBoxFunc(&buttonboxfunc);
	glutDialsFunc(&dialsfunc);
	glutTabletMotionFunc(&tabletmotionfunc);
	glutTabletButtonFunc(&tabletbuttonfunc);
}

LTL_METHOD_DECL(GLUT_MetaLibrary_init_0)
{
	int vmargc = little_system_vmargc();
	char **vmargv = little_system_vmargv();
	printf("GLUT_MetaLibrary_init_0\n");
	glut_threadenv = threadenv;
	painter_mid = ltl_threadenv_getsym(threadenv,"display_0");
	keyboard_mid = ltl_threadenv_getsym(threadenv,"keyboard_3");
	mouse_mid = ltl_threadenv_getsym(threadenv,"mouse_4");
	motion_mid = ltl_threadenv_getsym(threadenv,"motion_2");
	passivemotion_mid = ltl_threadenv_getsym(threadenv,"passiveMotion_2");
	reshaper_mid = ltl_threadenv_getsym(threadenv,"reshape_2");
	entry_mid = ltl_threadenv_getsym(threadenv,"entry_1");
	special_mid = ltl_threadenv_getsym(threadenv,"special_3");
	spaceballmotion_mid = ltl_threadenv_getsym(threadenv,"spaceballMotion_3");
	spaceballrotate_mid = ltl_threadenv_getsym(threadenv,"spaceballRotate_3");
	spaceballbutton_mid = ltl_threadenv_getsym(threadenv,"spaceballButton_2");
	buttonbox_mid = ltl_threadenv_getsym(threadenv,"buttonBox_2");
	dials_mid = ltl_threadenv_getsym(threadenv,"dials_2");
	tabletmotion_mid = ltl_threadenv_getsym(threadenv,"tabletMotion_2");
	tabletbutton_mid = ltl_threadenv_getsym(threadenv,"tabletButton_4");
	menustate_mid = ltl_threadenv_getsym(threadenv,"menuStateCB_3");
	idle_mid = ltl_threadenv_getsym(threadenv,"idleCB_0");
	timer_mid = ltl_threadenv_getsym(threadenv,"time_1");
	menu_mid = ltl_threadenv_getsym(threadenv,"menu_1");
	LTL_ASSERT_CLASS(char_class,CHAR_CLASS);
	LTL_ASSERT_CLASS(float_class,FLOAT_CLASS);
	LTL_ASSERT_CLASS(hmousebutton_class,HMOUSEBUTTON_CLASS);
	LTL_ASSERT_CLASS(hbitmapfont_class,HBITMAPFONT_CLASS);
	LTL_ASSERT_CLASS(hstrokefont_class,HSTROKEFONT_CLASS);
	LTL_ASSERT_CLASS(hmenu_class,HMENU_CLASS);
	LTL_ASSERT_CLASS(hrgbcomp_class,HRGBCOMP_CLASS);
	LTL_ASSERT_CLASS(hwindow_class,HWINDOW_CLASS);
	LTL_ASSERT_CLASS(strbuf_class,STRBUF_CLASS);
	glutInit(&vmargc,vmargv);
	initwrappers(threadenv);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__init_0)
{
	ltl_sym_t heap_class = -1;
	printf("GLUT_Metaglut__init_0\n");
	glut_obj = ltl_threadenv_get_global4sym(threadenv,ltl_threadenv_getsym(threadenv,"GLUT/glut"));
	if ( (painter_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,painter_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no display_0 method","");
	}
	if ( (keyboard_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,keyboard_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no keyboard_3 method","");
	}
	if ( (mouse_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,mouse_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no mouse_4 method","");
	}
	if ( (motion_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,motion_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no motion_2 method","");
	}
	if ( (passivemotion_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,passivemotion_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no passiveMotion_2 method","");
	}
	if ( (reshaper_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,reshaper_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no reshape_2 method","");
	}
	if ( (entry_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,entry_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no entry_1 method","");
	}
	if ( (special_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,special_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no special_3 method","");
	}
	if ( (spaceballmotion_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,spaceballmotion_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no spaceballmotion_3 method","");
	}
	if ( (spaceballrotate_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,spaceballrotate_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no spaceballrotate_3 method","");
	}
	if ( (spaceballbutton_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,spaceballbutton_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no spaceballbutton_2 method","");
	}
	if ( (buttonbox_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,buttonbox_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no buttonBox_2 method","");
	}
	if ( (dials_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,dials_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no dials_2 method","");
	}
	if ( (tabletmotion_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,tabletmotion_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no tabletmotion_2 method","");
	}
	if ( (tabletbutton_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,tabletbutton_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no tabletbutton_4 method","");
	}
	if ( (menu_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,menu_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no menu_1 method","");
	}
	if ( (timer_callback = ltl_threadenv_get_method4tf(glut_threadenv,glut_obj->type,timer_mid)) == NULL ) {
		LITTLE_ERROR(threadenv,"NoSuchMethod","GLUT has no time_1 method","");
	}
	LTL_ASSERT_CLASS(heap_class,HEAP_CLASS);
	ltl_gc_lock(&computer);
	ltl_gc_lock(&menustate);
	ltl_gc_lock(&timer);
	return self;
}

LTL_METHOD_DECL(GLUT_MetaGLUT_LEFT_BUTTON_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hmousebutton_class,GLUT_LEFT_BUTTON);
	return ret;
}

LTL_METHOD_DECL(GLUT_MetaGLUT_MIDDLE_BUTTON_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hmousebutton_class,GLUT_MIDDLE_BUTTON);
	return ret;
}

LTL_METHOD_DECL(GLUT_MetaGLUT_RIGHT_BUTTON_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hmousebutton_class,GLUT_RIGHT_BUTTON);
	return ret;
}

#ifdef GLUT_BITMAP_8_BY_13
LTL_METHOD_DECL(GLUT_MetaGLUT_BITMAP_8_BY_13_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hbitmapfont_class,GLUT_BITMAP_8_BY_13);
	return ret;
}
#endif

#ifdef GLUT_BITMAP_9_BY_15
LTL_METHOD_DECL(GLUT_MetaGLUT_BITMAP_9_BY_15_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hbitmapfont_class,GLUT_BITMAP_9_BY_15);
	return ret;
}
#endif

#ifdef GLUT_BITMAP_HELVETICA_10
LTL_METHOD_DECL(GLUT_MetaGLUT_BITMAP_HELVETICA_10_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hbitmapfont_class,GLUT_BITMAP_HELVETICA_10);
	return ret;
}
#endif

#ifdef GLUT_BITMAP_HELVETICA_12
LTL_METHOD_DECL(GLUT_MetaGLUT_BITMAP_HELVETICA_12_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hbitmapfont_class,GLUT_BITMAP_HELVETICA_12);
	return ret;
}
#endif

#ifdef GLUT_BITMAP_HELVETICA_18
LTL_METHOD_DECL(GLUT_MetaGLUT_BITMAP_HELVETICA_18_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hbitmapfont_class,GLUT_BITMAP_HELVETICA_18);
	return ret;
}
#endif

#ifdef GLUT_BITMAP_TIMES_ROMAN_10
LTL_METHOD_DECL(GLUT_MetaGLUT_BITMAP_TIMES_ROMAN_10_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hbitmapfont_class,GLUT_BITMAP_TIMES_ROMAN_10);
	return ret;
}
#endif

#ifdef GLUT_BITMAP_TIMES_ROMAN_24
LTL_METHOD_DECL(GLUT_MetaGLUT_BITMAP_TIMES_ROMAN_24_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hbitmapfont_class,GLUT_BITMAP_TIMES_ROMAN_24);
	return ret;
}
#endif

#ifdef GLUT_STROKE_MONO_ROMAN
LTL_METHOD_DECL(GLUT_MetaGLUT_STROKE_MONO_ROMAN_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hstrokefont_class,GLUT_STROKE_MONO_ROMAN);
	return ret;
}
#endif

#ifdef GLUT_STROKE_ROMAN
LTL_METHOD_DECL(GLUT_MetaGLUT_STROKE_ROMAN_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hstrokefont_class,GLUT_STROKE_ROMAN);
	return ret;
}
#endif

#ifdef GLUT_RED
LTL_METHOD_DECL(GLUT_MetaGLUT_RED_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hrgbcomp_class,GLUT_RED);
	return ret;
}
#endif

#ifdef GLUT_GREEN
LTL_METHOD_DECL(GLUT_MetaGLUT_GREEN_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hrgbcomp_class,GLUT_GREEN);
	return ret;
}
#endif

#ifdef GLUT_BLUE
LTL_METHOD_DECL(GLUT_MetaGLUT_BLUE_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hrgbcomp_class,GLUT_BLUE);
	return ret;
}
#endif

LTL_METHOD_DECL(GLUT_Metaglut_initDisplayMode_1)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_GLUT_initDisplayMode_1");
	/*printf("GLUT_MetaGLUT_initDisplayMode_1: mode=%d\n",(int) argv[0].fields);*/
	glutInitDisplayMode((int) argv[0].fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut_initWindowPosition_2)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_GLUT_initPosition_2");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLUT_GLUT_initPosition_2");
	glutInitWindowPosition((int) argv[0].fields,(int) argv[1].fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut_initWindowSize_2)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_GLUT_initSize_2");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLUT_GLUT_initSize_2");
	glutInitWindowSize((int) argv[0].fields,(int) argv[1].fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut__mainLoop_0)
{
	glutMainLoop();
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut__createWindow_1)
{
	if ( glut_threadenv && glut_obj ) {
		char *name = LTL_STRBUF_PCHAR(argv);
		int handle = glutCreateWindow(name);
		ltl_object_t *ret = ltl_threadenv_push(threadenv);
		printf("GLUT_Metaglut_createWindow_1(): %d\n",handle);
		registerCallbacks();
		LTL_OBJECT_SETSPECIAL(ret,hwindow_class,handle);
		return ret;
	}
	else {
		return LTL_NIL;
	}
}

LTL_METHOD_DECL(GLUT_Metaglut_createWindow_5)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	int hwin = 0;
	int x, y, w, h;
	LTL_CHECK_ARG(0,hwindow_class,"GLUT_Metaglut_createWindow_5");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLUT_Metaglut_createWindow_5");
	LTL_CHECK_ARG(2,LTL_TYPE_WORD,"GLUT_Metaglut_createWindow_5");
	LTL_CHECK_ARG(3,LTL_TYPE_WORD,"GLUT_Metaglut_createWindow_5");
	LTL_CHECK_ARG(4,LTL_TYPE_WORD,"GLUT_Metaglut_createWindow_5");
	hwin = (int) argv[0].fields;
	x = (int) argv[1].fields;
	y = (int) argv[2].fields;
	w = (int) argv[3].fields;
	h = (int) argv[4].fields;
	LTL_OBJECT_SETSPECIAL(
		ret,
		hwindow_class,
		glutCreateSubWindow(hwin,x,y,w,h));
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut__destroyWindow_1)
{
	LTL_CHECK_ARG(0,hwindow_class,"GLUT_Metaglut_destroyWindow_1");
	printf("GLUT_Metaglut_destroyWindow_1(%d)\n",argv->fields);
	glutDestroyWindow((int) argv->fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut_postRedisplay_0)
{
	glutPostRedisplay();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_postRedisplay_1)
{
	LTL_CHECK_ARG(0,hwindow_class,"GLUT_Metaglut_postRedisplay_1");
	glutPostWindowRedisplay((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_swapBuffers_0)
{
	glutSwapBuffers();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_getWindow_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(
		ret,
		hwindow_class,
		glutGetWindow());
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_setWindow_1)
{
	LTL_CHECK_ARG(0,hwindow_class,"GLUT_Metaglut_setWindow_1");
	printf("GLUT_Metaglut_setWindow_1(%d)\n",argv->fields);
	glutSetWindow((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__setWindowTitle_1)
{
	LTL_CHECK_ARG(0,strbuf_class,"GLUT_Metaglut__setWindowTitle_1");
	glutSetWindowTitle(LTL_STRBUF_PCHAR(argv));
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__setIconTitle_1)
{
	LTL_CHECK_ARG(0,strbuf_class,"GLUT_Metaglut__setIconTitle_1");
	glutSetIconTitle(LTL_STRBUF_PCHAR(argv));
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_positionWindow_2)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_positionWindow_2");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLUT_Metaglut_positionWindow_2");
	glutPositionWindow((int) argv[0].fields,(int) argv[1].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_reshapeWindow_2)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_reshapeWindow_2");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLUT_Metaglut_reshapeWindow_2");
	glutReshapeWindow((int) argv[0].fields,(int) argv[1].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_popWindow_0)
{
	glutPopWindow();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_pushWindow_0)
{
	glutPushWindow();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_iconifyWindow_0)
{
	glutIconifyWindow();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_showWindow_0)
{
	glutShowWindow();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_hideWindow_0)
{
	glutHideWindow();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_fullScreen_0)
{
	glutFullScreen();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_establishOverlay_0)
{
	glutEstablishOverlay();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_removeOverlay_0)
{
	glutRemoveOverlay();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_useLayer_1)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_useLayer_1");
	glutUseLayer((GLenum) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_postOverlayRedisplay_0)
{
	glutPostOverlayRedisplay();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_postOverlayRedisplay_1)
{
	LTL_CHECK_ARG(0,hwindow_class,"GLUT_Metaglut_postOverlayRedisplay_1");
	glutPostWindowOverlayRedisplay((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_showOverlay_0)
{
	glutShowOverlay();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_hideOverlay_0)
{
	glutHideOverlay();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__createMenu_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hmenu_class,glutCreateMenu(&menufunc));
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut__destroyMenu_1)
{
	LTL_CHECK_ARG(0,hmenu_class,"GLUT_Metaglut_destroyMenu_1");
	glutDestroyMenu((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_getMenu_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,hmenu_class,glutGetMenu());
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_setMenu_1)
{
	LTL_CHECK_ARG(0,hmenu_class,"GLUT_Metaglut_setMenu_1");
	glutSetMenu((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__addMenuEntry_2)
{
	LTL_CHECK_ARG(0,strbuf_class,"GLUT_Metaglut__addMenuEntry_2");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLUT_Metaglut__addMenuEntry_2");
	glutAddMenuEntry(LTL_STRBUF_PCHAR(argv),(int) argv[1].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__addSubMenu_2)
{
	LTL_CHECK_ARG(0,strbuf_class,"GLUT_Metaglut__addSubMenu_2");
	LTL_CHECK_ARG(1,hmenu_class,"GLUT_Metaglut__addSubMenu_2");
	glutAddSubMenu(LTL_STRBUF_PCHAR(argv),(int) argv[1].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__changeToMenuEntry_3)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut__changeToMenuEntry_3");
	LTL_CHECK_ARG(1,strbuf_class,"GLUT_Metaglut__changeToMenuEntry_3");
	LTL_CHECK_ARG(2,LTL_TYPE_WORD,"GLUT_Metaglut__changeToMenuEntry_3");
	glutChangeToMenuEntry((int) argv[0].fields,LTL_STRBUF_PCHAR(argv+1),(int) argv[2].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut__changeToSubMenu_3)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut__changeToSubMenu_3");
	LTL_CHECK_ARG(1,strbuf_class,"GLUT_Metaglut__changeToSubMenu_3");
	LTL_CHECK_ARG(2,hmenu_class,"GLUT_Metaglut__changeToSubMenu_3");
	glutChangeToSubMenu((int) argv[0].fields,LTL_STRBUF_PCHAR(argv+1),(int) argv[2].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_removeMenuItem_1)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_removeMenuItem_1");
	glutRemoveMenuItem((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_attachMenu_1)
{
	LTL_CHECK_ARG(0,hmousebutton_class,"GLUT_Metaglut_attachMenu_1");
	glutAttachMenu((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_detachMenu_1)
{
	LTL_CHECK_ARG(0,hmousebutton_class,"GLUT_Metaglut_detachMenu_1");
	glutDetachMenu((int) argv->fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_idleFunc_1)
{
	/*if( computer.type ) {
		ltl_gc_unlock(&computer);
	}*/
	computer = argv[0];
	/*ltl_gc_lock(&computer);*/
	if ( computer.type ) {
		if ( (idle_callback = ltl_threadenv_get_method4tf(glut_threadenv,computer.type,idle_mid)) == NULL ) {
			LITTLE_ERROR(threadenv,"NoSuchMethod","computer class has no idleCB_0 method","");
		}
		glutIdleFunc(&idlefunc);
	}
	else {
		glutIdleFunc(NULL);
	}
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut_menuStateFunc_1)
{
	/*if ( menustate.type ) {
		ltl_gc_unlock(&menustate);
	}*/
	menustate = argv[0];
	/*ltl_gc_lock(&menustate);*/
	if ( menustate.type ) {
		if ( (menustate_callback = ltl_threadenv_get_method4tf(glut_threadenv,menustate.type,menustate_mid)) == NULL ) {
			LITTLE_ERROR(threadenv,"NoSuchMethod","menustate class has no menuStateCB_3 method","");
		}
		glutMenuStatusFunc(&menustatefunc);
	}
	else {
		glutMenuStatusFunc(NULL);
	}
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut__timerFunc_2)
{
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_timerFunc_1");
	LTL_CHECK_ARG(1,LTL_TYPE_WORD,"GLUT_Metaglut_timerFunc_1");
	glutTimerFunc((int)argv[0].fields,&timerfunc,(int)argv[2].fields);
	return LTL_NIL;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireSphere_3)
{
	double size = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_wireSphere_3");
	int slices = init_iscalar(threadenv,argv,1,"GLUT_Metaglut_wireSphere_3");
	int stacks = init_iscalar(threadenv,argv,2,"GLUT_Metaglut_wireSphere_3");
	glutWireSphere(size,slices,stacks);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidSphere_3)
{
	double size = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_solidSphere_3");
	int slices = init_iscalar(threadenv,argv,1,"GLUT_Metaglut_solidSphere_3");
	int stacks = init_iscalar(threadenv,argv,2,"GLUT_Metaglut_solidSphere_3");
	glutSolidSphere(size,slices,stacks);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireCone_4)
{
	double base = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_wireCone_4");
	double height = init_dscalar(threadenv,argv,1,"GLUT_Metaglut_wireCone_4");
	int slices = init_iscalar(threadenv,argv,2,"GLUT_Metaglut_wireSphere_3");
	int stacks = init_iscalar(threadenv,argv,3,"GLUT_Metaglut_wireSphere_3");
	glutWireCone(base,height,slices,stacks);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidCone_4)
{
	double base = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_solidCone_4");
	double height = init_dscalar(threadenv,argv,1,"GLUT_Metaglut_solidCone_4");
	int slices = init_iscalar(threadenv,argv,2,"GLUT_Metaglut_solidCone_4");
	int stacks = init_iscalar(threadenv,argv,3,"GLUT_Metaglut_solidCone_4");
	glutSolidCone(base,height,slices,stacks);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireCube_1)
{
	double size = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_wireTeapot_1");
	glutWireCube(size);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidCube_1)
{
	double size = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_wireTeapot_1");
	glutSolidCube(size);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireTorus_4)
{
	double inner = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_wireTorus_4");
	double outer = init_dscalar(threadenv,argv,1,"GLUT_Metaglut_wireTorus_4");
	int sides = init_iscalar(threadenv,argv,2,"GLUT_Metaglut_wireTorus_4");
	int rings = init_iscalar(threadenv,argv,3,"GLUT_Metaglut_wireTorus_4");
	glutWireTorus(inner,outer,sides,rings);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidTorus_4)
{
	double inner = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_solidTorus_4");
	double outer = init_dscalar(threadenv,argv,1,"GLUT_Metaglut_solidTorus_4");
	int sides = init_iscalar(threadenv,argv,2,"GLUT_Metaglut_solidTorus_4");
	int rings = init_iscalar(threadenv,argv,3,"GLUT_Metaglut_solidTorus_4");
	glutSolidTorus(inner,outer,sides,rings);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireDodecahedron_0)
{
	glutWireDodecahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidDodecahedron_0)
{
	glutSolidDodecahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireOctahedron_0)
{
	glutWireOctahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidOctahedron_0)
{
	glutSolidOctahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireTeapot_1)
{
	double size = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_wireTeapot_1");
	glutWireTeapot(size);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidTeapot_1)
{
	double size = init_dscalar(threadenv,argv,0,"GLUT_Metaglut_wireTeapot_1");
	glutSolidTeapot(size);
	return self;
}


LTL_METHOD_DECL(GLUT_Metaglut_wireTetrahedron_0)
{
	glutWireTetrahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidTetrahedron_0)
{
	glutSolidTetrahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_wireIcosahedron_0)
{
	glutWireIcosahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_solidIcosahedron_0)
{
	glutSolidIcosahedron();
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_setColor_4)
{
	float r = init_fscalar(threadenv,argv,1,"GLUT_Metaglut_setColor_4");
	float g = init_fscalar(threadenv,argv,2,"GLUT_Metaglut_setColor_4");
	float b = init_fscalar(threadenv,argv,3,"GLUT_Metaglut_setColor_4");
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_setColor_4");
	glutSetColor((int) argv[0].fields,r,g,b);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_getColor_2)
{
	union {
		float			f;
		ltl_object_t	*o;
	} val;
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_getColor_2");
	LTL_CHECK_ARG(1,hrgbcomp_class,"GLUT_Metaglut_getColor_2");
	val.f = glutGetColor((int) argv[0].fields, (int) argv[1].fields);
	LTL_OBJECT_SETSPECIAL(ret,float_class,val.o);
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_copyColormap_1)
{
	LTL_CHECK_ARG(0,hwindow_class,"GLUT_Metaglut_copyColormap_1");
	glutCopyColormap((int) argv[0].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_bitmapCharacter_2)
{
	LTL_CHECK_ARG(0,hbitmapfont_class,"GLUT_Metaglut_bitmapCharacter_2");
	LTL_CHECK_ARG(1,char_class,"GLUT_Metaglut_bitmapCharacter_2");
	glutBitmapCharacter((void *) argv[0].fields,(int) argv[1].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_bitmapWitdh_2)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_CHECK_ARG(0,hbitmapfont_class,"GLUT_Metaglut_bitmapWitdh_2");
	LTL_CHECK_ARG(1,char_class,"GLUT_Metaglut_bitmapWitdh_2");
	LTL_OBJECT_SETSPECIAL(
		ret,
		LTL_TYPE_WORD,
		glutBitmapWidth((void *) argv[0].fields,(int) argv[1].fields));
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_strokeCharacter_2)
{
	LTL_CHECK_ARG(0,hstrokefont_class,"GLUT_Metaglut_strokeCharacter_2");
	LTL_CHECK_ARG(1,char_class,"GLUT_Metaglut_strokeCharacter_2");
	glutStrokeCharacter((void *) argv[0].fields,(int) argv[1].fields);
	return self;
}

LTL_METHOD_DECL(GLUT_Metaglut_strokeWitdh_2)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_CHECK_ARG(0,hstrokefont_class,"GLUT_Metaglut_strokeWitdh_2");
	LTL_CHECK_ARG(1,char_class,"GLUT_Metaglut_strokeWitdh_2");
	LTL_OBJECT_SETSPECIAL(
		ret,
		LTL_TYPE_WORD,
		glutStrokeWidth((void *) argv[0].fields,(int) argv[1].fields));
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_get_1)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_get_1");
	LTL_OBJECT_SETSPECIAL(ret,LTL_TYPE_WORD,glutGet((GLenum) argv[0].fields));
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_layerGet_1)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_layerGet_1");
	LTL_OBJECT_SETSPECIAL(ret,LTL_TYPE_WORD,glutLayerGet((GLenum) argv[0].fields));
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_deviceGet_1)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_CHECK_ARG(0,LTL_TYPE_WORD,"GLUT_Metaglut_deviceGet_1");
	LTL_OBJECT_SETSPECIAL(ret,LTL_TYPE_WORD,glutDeviceGet((GLenum) argv[0].fields));
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut_getModifiers_0)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_OBJECT_SETSPECIAL(ret,LTL_TYPE_WORD,glutGetModifiers());
	return ret;
}

LTL_METHOD_DECL(GLUT_Metaglut__extensionSupported_1)
{
	ltl_object_t *ret = ltl_threadenv_push(threadenv);
	LTL_CHECK_ARG(0,strbuf_class,"GLUT_Metaglut__extensionSupported_1");
	LTL_OBJECT_SETSPECIAL(ret,LTL_TYPE_WORD,glutExtensionSupported(LTL_STRBUF_PCHAR(argv)));
	return ret;
}
