#include <little/little_rt.h>
#include <little/wrapperutil.h>

#define PACKEDARRAY_CLASS	"little/lang/PackedArray"
static ltl_sym_t float_class		= -1;
static ltl_sym_t packedarray_class	= -1;

void initwrappers(ltl_threadenv_t *threadenv)
{
	LTL_ASSERT_CLASS(float_class,FLOAT_CLASS);
	LTL_ASSERT_CLASS(packedarray_class,PACKEDARRAY_CLASS);
}

int init_iscalar(ltl_threadenv_t *threadenv,ltl_object_t *argv,int n,const char *name)
{
	LTL_CHECK_ARG(n,LTL_TYPE_WORD,name);
	return (int) argv[n].fields;
}
		
float init_fscalar(ltl_threadenv_t *threadenv,ltl_object_t *argv,int n,const char *name)
{
	union {
		float           f;
		ltl_object_t    *o;
	} val;
	LTL_CHECK_ARG(n,float_class,name);
	val.o = argv[n].fields;
	return val.f;
}

double init_dscalar(ltl_threadenv_t *threadenv,ltl_object_t *argv,int n,const char *name)
{
	union {
		float           f;
		ltl_object_t    *o;
	} val;
	LTL_CHECK_ARG(n,float_class,name);
	val.o = argv[n].fields;
	return (double) val.f;
}

int *init_ivec(ltl_threadenv_t *threadenv,ltl_object_t *argv,int n,const char *name)
{
	ltl_packed_array_t *array = (ltl_packed_array_t *)argv[n].fields;
	LTL_CHECK_ARG(n,packedarray_class,name);
	if ( ltl_packed_array_type(array) != LTL_TYPE_WORD ) {
		LITTLE_TYPEMISMATCH(
			threadenv,
			ltl_threadenv_get_str4sym(threadenv,LTL_TYPE_WORD),
			ltl_threadenv_get_str4sym(threadenv,ltl_packed_array_type(array)));
	}
	return (int *) ltl_packed_array_at((ltl_packed_array_t *)argv[n].fields,0);
}

float *init_fvec(ltl_threadenv_t *threadenv,ltl_object_t *argv,int n,const char *name)
{
	ltl_packed_array_t *array = (ltl_packed_array_t *)argv[n].fields;
	LTL_CHECK_ARG(n,packedarray_class,name);
	if ( ltl_packed_array_type(array) != float_class ) {
		LITTLE_TYPEMISMATCH(
			threadenv,
			ltl_threadenv_get_str4sym(threadenv,float_class),
			ltl_threadenv_get_str4sym(threadenv,ltl_packed_array_type(array)));
	}
	return (float *) ltl_packed_array_at((ltl_packed_array_t *)argv[n].fields,0);
}
